@@ -12,10 +12,11 @@ OQS_SIG *OQS_SIG_cross_rsdp_192_balanced_new(void) {
 		return NULL;
 	}
 	sig->method_name = OQS_SIG_alg_cross_rsdp_192_balanced;
-	sig->alg_version = "1.2 + Keccak_x4 + PQClean fixes";
+	sig->alg_version = "2.0 + PQClean and OQS patches";
 
 	sig->claimed_nist_level = 3;
 	sig->euf_cma = true;
+	sig->suf_cma = false;
 	sig->sig_with_ctx_support = false;
 
 	sig->length_public_key = OQS_SIG_cross_rsdp_192_balanced_length_public_key;
