@@ -16,6 +16,7 @@
 #include "system_info.c"
 
 #define RNDBYTES 32
+#define MAXCTXBYTES 255
 
 #ifdef OQS_ENABLE_SIG_ml_dsa_44
 extern int pqcrystals_ml_dsa_44_ref_signature_internal(uint8_t *sig,
@@ -147,7 +148,6 @@ static OQS_STATUS sig_kg_vector(const char *method_name,
 	OQS_SIG *sig = NULL;
 	uint8_t *public_key = NULL;
 	uint8_t *secret_key = NULL;
-	uint8_t *signature = NULL;
 	OQS_STATUS rc, ret = OQS_ERROR;
 
 	void (*randombytes_init)(const uint8_t *, const uint8_t *) = NULL;
@@ -175,8 +175,7 @@ static OQS_STATUS sig_kg_vector(const char *method_name,
 
 	public_key = OQS_MEM_malloc(sig->length_public_key);
 	secret_key = OQS_MEM_malloc(sig->length_secret_key);
-	signature = OQS_MEM_malloc(sig->length_signature);
-	if ((public_key == NULL) || (secret_key == NULL) || (signature == NULL)) {
+	if ((public_key == NULL) || (secret_key == NULL)) {
 		fprintf(stderr, "[vectors_sig] %s ERROR: OQS_MEM_malloc failed!\n", method_name);
 		goto err;
 	}
@@ -217,7 +216,6 @@ cleanup:
 		randombytes_free();
 	}
 	OQS_MEM_insecure_free(public_key);
-	OQS_MEM_insecure_free(signature);
 	OQS_SIG_free(sig);
 	return ret;
 }
@@ -286,9 +284,61 @@ cleanup:
 
 }
 
+static int sig_ver_vector_ext(const char *method_name,
+                              const uint8_t *sigVer_pk_bytes,
+                              const uint8_t *sigVer_msg_bytes,
+                              size_t msgLen,
+                              const uint8_t *sigVer_sig_bytes,
+                              const uint8_t *sigVer_ctx, size_t sigVer_ctxLen,
+                              int testPassed) {
+
+	FILE *fh = NULL;
+	OQS_SIG *sig = NULL;
+	int rc = -1, ret = -1;
+
+	if ((sigVer_pk_bytes == NULL) || (sigVer_msg_bytes == NULL) || (sigVer_sig_bytes == NULL) || (sigVer_ctxLen && sigVer_ctx == NULL)) {
+		fprintf(stderr, "[vectors_sig] %s ERROR: inputs NULL!\n", method_name);
+		goto err;
+	}
+
+	sig = OQS_SIG_new(method_name);
+	if (sig == NULL) {
+		printf("[vectors_sig] %s was not enabled at compile-time.\n", method_name);
+		goto algo_not_enabled;
+	}
+
+	fh = stdout;
+
+	rc = OQS_SIG_verify_with_ctx_str(sig, sigVer_msg_bytes, msgLen, sigVer_sig_bytes, sig->length_signature, sigVer_ctx, sigVer_ctxLen, sigVer_pk_bytes);
+
+	if ((!rc) != testPassed) {
+		fprintf(stderr, "[vectors_sig] %s ERROR: OQS_SIG_verify_with_ctx_str failed!\n", method_name);
+		goto err;
+	} else {
+		ret = EXIT_SUCCESS;
+		printf("success\n");
+	}
+
+	fprintBstr(fh, "testPassed: ", (const uint8_t *)&testPassed, 1);
+
+	goto cleanup;
+
+err:
+	ret = EXIT_FAILURE;
+	goto cleanup;
+
+algo_not_enabled:
+	ret = EXIT_SUCCESS;
+
+cleanup:
+	OQS_SIG_free(sig);
+	return ret;
+
+}
+
 static int sig_gen_vector(const char *method_name,
                           uint8_t *prng_output_stream,
-                          const uint8_t *sigGen_sk, const uint8_t *sigGen_msg, size_t sigGen_msgLen, const uint8_t *sigGen_sig, int randomized) {
+                          const uint8_t *sigGen_sk, const uint8_t *sigGen_msg, size_t sigGen_msgLen, const uint8_t *sigGen_sig) {
 
 	FILE *fh = NULL;
 	uint8_t *signature = NULL;
@@ -313,7 +363,7 @@ static int sig_gen_vector(const char *method_name,
 		goto err;
 	}
 
-	if ((randomized && prng_output_stream == NULL) || (sigGen_sk == NULL) || (sigGen_msg == NULL) || (sigGen_sig == NULL)) {
+	if ((prng_output_stream == NULL) || (sigGen_sk == NULL) || (sigGen_msg == NULL) || (sigGen_sig == NULL)) {
 		fprintf(stderr, "[vectors_sig] %s ERROR: inputs NULL!\n", method_name);
 		goto err;
 	}
@@ -338,13 +388,13 @@ static int sig_gen_vector(const char *method_name,
 		fprintf(stderr, "[vectors_sig] %s ERROR: ml_dsa_sign_internal failed!\n", method_name);
 		goto err;
 	}
-	fprintBstr(fh, "signature: ", signature, sig->length_public_key);
+	fprintBstr(fh, "signature: ", signature, sig->length_signature);
 
 	if (!memcmp(signature, sigGen_sig, sigLen)) {
 		ret = EXIT_SUCCESS;
 	} else {
 		ret = EXIT_FAILURE;
-		fprintf(stderr, "[vectors_sig] %s ERROR: public key or private key doesn't match!\n", method_name);
+		fprintf(stderr, "[vectors_sig] %s ERROR: signature doesn't match!\n", method_name);
 	}
 	goto cleanup;
 
@@ -360,6 +410,87 @@ cleanup:
 	OQS_SIG_free(sig);
 	return ret;
 }
+
+static int sig_gen_vector_ext(const char *method_name,
+                              uint8_t *prng_output_stream,
+                              const uint8_t *sigGen_sk, const uint8_t *sigGen_msg, size_t sigGen_msgLen,
+                              const uint8_t *sigGen_ctx, size_t sigGen_ctxLen, const uint8_t *sigGen_sig) {
+
+	uint8_t *entropy_input;
+	FILE *fh = NULL;
+	uint8_t *signature = NULL;
+	OQS_SIG *sig = NULL;
+	int rc = -1, ret = -1;
+	size_t sigLen;
+
+	void (*randombytes_init)(const uint8_t *, const uint8_t *) = NULL;
+	void (*randombytes_free)(void) = NULL;
+
+	sig = OQS_SIG_new(method_name);
+	if (sig == NULL) {
+		printf("[vectors_sig] %s was not enabled at compile-time.\n", method_name);
+		goto algo_not_enabled;
+	}
+
+	if (is_ml_dsa(method_name)) {
+		OQS_randombytes_custom_algorithm(&MLDSA_randombytes);
+		randombytes_init = &MLDSA_randombytes_init;
+		randombytes_free = &MLDSA_randombytes_free;
+		entropy_input = (uint8_t *) prng_output_stream;
+	} else {
+		// Only ML-DSA supported
+		goto err;
+	}
+
+	randombytes_init(entropy_input, NULL);
+
+	sigLen = sig->length_signature;
+
+	fh = stdout;
+
+	signature = OQS_MEM_malloc(sigLen);
+
+	if (signature == NULL) {
+		fprintf(stderr, "[vectors_sig] %s ERROR: OQS_MEM_malloc failed!\n", method_name);
+		goto err;
+	}
+
+	if ((prng_output_stream == NULL) || (sigGen_sk == NULL) || (sigGen_msg == NULL) || (sigGen_sig == NULL) || (sigGen_ctxLen && sigGen_ctx == NULL)) {
+		fprintf(stderr, "[vectors_sig] %s ERROR: inputs NULL!\n", method_name);
+		goto err;
+	}
+
+	rc = OQS_SIG_sign_with_ctx_str(sig, signature, &sigLen, sigGen_msg, sigGen_msgLen, sigGen_ctx, sigGen_ctxLen, sigGen_sk);
+
+	if (rc) {
+		fprintf(stderr, "[vectors_sig] %s ERROR: OQS_SIG_sign_with_ctx_str failed!\n", method_name);
+		goto err;
+	}
+	fprintBstr(fh, "signature: ", signature, sig->length_signature);
+
+	if (!memcmp(signature, sigGen_sig, sigLen)) {
+		ret = EXIT_SUCCESS;
+	} else {
+		ret = EXIT_FAILURE;
+		fprintf(stderr, "[vectors_sig] %s ERROR: signature doesn't match!\n", method_name);
+	}
+	goto cleanup;
+
+err:
+	ret = EXIT_FAILURE;
+	goto cleanup;
+
+algo_not_enabled:
+	ret = EXIT_SUCCESS;
+
+cleanup:
+	OQS_MEM_insecure_free(signature);
+	if (randombytes_free != NULL) {
+		randombytes_free();
+	}
+	OQS_SIG_free(sig);
+	return ret;
+}
 #endif
 
 
@@ -371,6 +502,7 @@ int main(int argc, char **argv) {
 	OQS_init();
 
 	size_t msgLen;
+	size_t ctxlen;
 
 	char *prng_output_stream = NULL;
 	char *kg_pk;
@@ -379,10 +511,12 @@ int main(int argc, char **argv) {
 	char *sigGen_sk;
 	char *sigGen_msg;
 	char *sigGen_sig;
+	char *sigGen_ctx;
 
 	char *sigVer_pk;
 	char *sigVer_msg;
 	char *sigVer_sig;
+	char *sigVer_ctx;
 
 	uint8_t *prng_output_stream_bytes = NULL;
 	uint8_t *kg_pk_bytes = NULL;
@@ -391,11 +525,12 @@ int main(int argc, char **argv) {
 	uint8_t *sigGen_sk_bytes = NULL;
 	uint8_t *sigGen_msg_bytes = NULL;
 	uint8_t *sigGen_sig_bytes = NULL;
-
+	uint8_t *sigGen_ctx_bytes = NULL;
 
 	uint8_t *sigVer_pk_bytes = NULL;
 	uint8_t *sigVer_msg_bytes = NULL;
 	uint8_t *sigVer_sig_bytes = NULL;
+	uint8_t *sigVer_ctx_bytes = NULL;
 
 	OQS_SIG *sig = NULL;
 
@@ -419,11 +554,11 @@ int main(int argc, char **argv) {
 			valid_args = false;
 			goto err;
 		}
-		prng_output_stream = argv[3]; // d || z
+		prng_output_stream = argv[3];
 		kg_pk = argv[4];
 		kg_sk = argv[5];
 
-		if (strlen(prng_output_stream) % 2 != 0 ||
+		if (strlen(prng_output_stream) != 2 * RNDBYTES ||
 		        strlen(kg_pk) != 2 * sig->length_public_key ||
 		        strlen(kg_sk) != 2 * sig->length_secret_key) {
 			printf("lengths bad\n");
@@ -446,30 +581,21 @@ int main(int argc, char **argv) {
 
 		rc = sig_kg_vector(alg_name, prng_output_stream_bytes, kg_pk_bytes, kg_sk_bytes);
 
-	} else if (!strcmp(test_name, "sigGen_det") || !strcmp(test_name, "sigGen_rnd")) {
-		int randomized = !strcmp(test_name, "sigGen_rnd");
-		if (argc != 6 + randomized) {
+	} else if (!strcmp(test_name, "sigGen_int")) {
+		if (argc != 7) {
 			valid_args = false;
 			goto err;
 		}
 		sigGen_sk = argv[3];
 		sigGen_msg = argv[4];
 		sigGen_sig = argv[5];
-
-		if (randomized) {
-			prng_output_stream = argv[6];
-			if (strlen(prng_output_stream) % 2 != 0) {
-				goto err;
-			}
-			prng_output_stream_bytes = OQS_MEM_malloc(strlen(prng_output_stream) / 2);
-			if (prng_output_stream_bytes == NULL) {
-				fprintf(stderr, "[vectors_sig] ERROR: OQS_MEM_malloc failed!\n");
-				goto err;
-			}
-		}
+		prng_output_stream = argv[6];
 
 		if ( strlen(sigGen_msg) % 2 != 0 ||
-		        strlen(sigGen_sig) != 2 * sig->length_signature) {
+		        strlen(prng_output_stream) != 2 * RNDBYTES ||
+		        strlen(sigGen_sig) != 2 * sig->length_signature ||
+		        strlen(sigGen_sk) != 2 * sig->length_secret_key) {
+			printf("lengths bad\n");
 			goto err;
 		}
 
@@ -478,32 +604,78 @@ int main(int argc, char **argv) {
 		sigGen_sk_bytes = OQS_MEM_malloc(sig->length_secret_key);
 		sigGen_msg_bytes = OQS_MEM_malloc(msgLen);
 		sigGen_sig_bytes = OQS_MEM_malloc(sig->length_signature);
+		prng_output_stream_bytes = OQS_MEM_malloc(RNDBYTES);
 
-		if ((sigGen_msg_bytes == NULL) || (sigGen_sig_bytes == NULL)) {
+		if ((sigGen_sk_bytes == NULL) || (sigGen_msg_bytes == NULL) || (sigGen_sig_bytes == NULL) || (prng_output_stream_bytes == NULL)) {
 			fprintf(stderr, "[vectors_sig] ERROR: OQS_MEM_malloc failed!\n");
 			goto err;
 		}
 
-		if (randomized) {
-			hexStringToByteArray(prng_output_stream, prng_output_stream_bytes);
-		} else {
-			prng_output_stream_bytes = OQS_MEM_malloc(32);
-			memset(prng_output_stream_bytes, 0, 32);
-		}
-
-
 		hexStringToByteArray(sigGen_sk, sigGen_sk_bytes);
 		hexStringToByteArray(sigGen_msg, sigGen_msg_bytes);
 		hexStringToByteArray(sigGen_sig, sigGen_sig_bytes);
+		hexStringToByteArray(prng_output_stream, prng_output_stream_bytes);
 
 #if defined(OQS_ENABLE_SIG_ml_dsa_44) || defined(OQS_ENABLE_SIG_ml_dsa_65) || defined(OQS_ENABLE_SIG_ml_dsa_87)
-		rc = sig_gen_vector(alg_name, prng_output_stream_bytes, sigGen_sk_bytes, sigGen_msg_bytes, msgLen, sigGen_sig_bytes, randomized);
+		rc = sig_gen_vector(alg_name, prng_output_stream_bytes, sigGen_sk_bytes, sigGen_msg_bytes, msgLen, sigGen_sig_bytes);
 #else
 		rc = EXIT_SUCCESS;
 		goto cleanup;
 #endif
 
-	} else if (!strcmp(test_name, "sigVer")) {
+	} else if (!strcmp(test_name, "sigGen_ext")) {
+		if (argc != 8) {
+			valid_args = false;
+			goto err;
+		}
+		sigGen_sk = argv[3];
+		sigGen_msg = argv[4];
+		sigGen_sig = argv[5];
+		sigGen_ctx = argv[6];
+		prng_output_stream = argv[7];
+
+		if ( strlen(sigGen_msg) % 2 != 0 ||
+		        strlen(sigGen_sig) != 2 * sig->length_signature ||
+		        strlen(prng_output_stream) != 2 * RNDBYTES ||
+		        strlen(sigGen_sk) != 2 * sig->length_secret_key ||
+		        strlen(sigGen_ctx) > 2 * MAXCTXBYTES) {
+			printf("lengths bad\n");
+			goto err;
+		}
+
+		msgLen = strlen(sigGen_msg) / 2;
+		ctxlen = strlen(sigGen_ctx) / 2;
+
+		sigGen_sk_bytes = OQS_MEM_malloc(sig->length_secret_key);
+		sigGen_msg_bytes = OQS_MEM_malloc(msgLen);
+		sigGen_sig_bytes = OQS_MEM_malloc(sig->length_signature);
+		prng_output_stream_bytes = OQS_MEM_malloc(RNDBYTES);
+		/* allocate memory if required */
+		if (ctxlen) {
+			sigGen_ctx_bytes = OQS_MEM_malloc(ctxlen);
+		}
+
+		if ((sigGen_sk_bytes == NULL) || (sigGen_msg_bytes == NULL) || (sigGen_sig_bytes == NULL) || (ctxlen && sigGen_ctx_bytes == NULL) || (prng_output_stream_bytes == NULL)) {
+			fprintf(stderr, "[vectors_sig] ERROR: OQS_MEM_malloc failed!\n");
+			goto err;
+		}
+
+		hexStringToByteArray(prng_output_stream, prng_output_stream_bytes);
+		hexStringToByteArray(sigGen_sk, sigGen_sk_bytes);
+		hexStringToByteArray(sigGen_msg, sigGen_msg_bytes);
+		hexStringToByteArray(sigGen_sig, sigGen_sig_bytes);
+		if (ctxlen) {
+			hexStringToByteArray(sigGen_ctx, sigGen_ctx_bytes);
+		}
+
+#if defined(OQS_ENABLE_SIG_ml_dsa_44) || defined(OQS_ENABLE_SIG_ml_dsa_65) || defined(OQS_ENABLE_SIG_ml_dsa_87)
+		rc = sig_gen_vector_ext(alg_name, prng_output_stream_bytes, sigGen_sk_bytes, sigGen_msg_bytes, msgLen, sigGen_ctx_bytes, ctxlen, sigGen_sig_bytes);
+#else
+		rc = EXIT_SUCCESS;
+		goto cleanup;
+#endif
+
+	} else if (!strcmp(test_name, "sigVer_int")) {
 		if (argc != 7) {
 			valid_args = false;
 			goto err;
@@ -518,6 +690,7 @@ int main(int argc, char **argv) {
 		        strlen(sigVer_sig) != 2 * sig->length_signature ||
 		        strlen(sigVer_pk) != 2 * sig->length_public_key ||
 		        (sigVerPassed != 0 && sigVerPassed != 1)) {
+			printf("lengths bad or incorrect verification status \n");
 			goto err;
 		}
 
@@ -527,6 +700,11 @@ int main(int argc, char **argv) {
 		sigVer_msg_bytes = OQS_MEM_malloc(msgLen);
 		sigVer_sig_bytes = OQS_MEM_malloc(sig->length_signature);
 
+		if ((sigVer_pk_bytes == NULL) || (sigVer_msg_bytes == NULL) || (sigVer_sig_bytes == NULL)) {
+			fprintf(stderr, "[vectors_sig] ERROR: OQS_MEM_malloc failed!\n");
+			goto err;
+		}
+
 		hexStringToByteArray(sigVer_pk, sigVer_pk_bytes);
 		hexStringToByteArray(sigVer_msg, sigVer_msg_bytes);
 		hexStringToByteArray(sigVer_sig, sigVer_sig_bytes);
@@ -538,8 +716,59 @@ int main(int argc, char **argv) {
 		goto cleanup;
 #endif
 
+	} else if (!strcmp(test_name, "sigVer_ext")) {
+		if (argc != 8) {
+			valid_args = false;
+			goto err;
+		}
+		sigVer_pk = argv[3];
+		sigVer_msg = argv[4];
+		sigVer_sig = argv[5];
+		sigVer_ctx = argv[6];
+
+		int sigVerPassed = atoi(argv[7]);
+
+		if ( strlen(sigVer_msg) % 2 != 0 ||
+		        strlen(sigVer_sig) != 2 * sig->length_signature ||
+		        strlen(sigVer_pk) != 2 * sig->length_public_key ||
+		        strlen(sigVer_ctx) > 2 * MAXCTXBYTES ||
+		        (sigVerPassed != 0 && sigVerPassed != 1)) {
+			printf("lengths bad or incorrect verification status \n");
+			goto err;
+		}
+
+		msgLen = strlen(sigVer_msg) / 2;
+		ctxlen = strlen(sigVer_ctx) / 2;
+
+		sigVer_pk_bytes = OQS_MEM_malloc(sig->length_public_key);
+		sigVer_msg_bytes = OQS_MEM_malloc(msgLen);
+		sigVer_sig_bytes = OQS_MEM_malloc(sig->length_signature);
+		/* allocate memory if required */
+		if (ctxlen) {
+			sigVer_ctx_bytes = OQS_MEM_malloc(ctxlen);
+		}
+
+		if ((sigVer_pk_bytes == NULL) || (sigVer_msg_bytes == NULL) || (sigVer_sig_bytes == NULL) || (ctxlen && sigVer_ctx_bytes == NULL)) {
+			fprintf(stderr, "[vectors_sig] ERROR: OQS_MEM_malloc failed!\n");
+			goto err;
+		}
+
+		hexStringToByteArray(sigVer_pk, sigVer_pk_bytes);
+		hexStringToByteArray(sigVer_msg, sigVer_msg_bytes);
+		hexStringToByteArray(sigVer_sig, sigVer_sig_bytes);
+		if (ctxlen) {
+			hexStringToByteArray(sigVer_ctx, sigVer_ctx_bytes);
+		}
+
+#if defined(OQS_ENABLE_SIG_ml_dsa_44) || defined(OQS_ENABLE_SIG_ml_dsa_65) || defined(OQS_ENABLE_SIG_ml_dsa_87)
+		rc = sig_ver_vector_ext(alg_name, sigVer_pk_bytes, sigVer_msg_bytes, msgLen, sigVer_sig_bytes, sigVer_ctx_bytes, ctxlen, sigVerPassed);
+#else
+		rc = EXIT_SUCCESS;
+		goto cleanup;
+#endif
+
 	} else {
-		printf("[vectors_sig] %s only keyGen/sigGen/sigVer supported!\n", alg_name);
+		printf("[vectors_sig] %s only keyGen/sigGen_int/sigGen_ext/sigVer_int/sigVer_ext supported!\n", alg_name);
 		goto err;
 	}
 	goto cleanup;
@@ -560,9 +789,11 @@ cleanup:
 	OQS_MEM_insecure_free(sigVer_pk_bytes);
 	OQS_MEM_insecure_free(sigVer_msg_bytes);
 	OQS_MEM_insecure_free(sigVer_sig_bytes);
+	OQS_MEM_insecure_free(sigVer_ctx_bytes);
 	OQS_MEM_insecure_free(sigGen_sk_bytes);
 	OQS_MEM_insecure_free(sigGen_msg_bytes);
 	OQS_MEM_insecure_free(sigGen_sig_bytes);
+	OQS_MEM_insecure_free(sigGen_ctx_bytes);
 	OQS_SIG_free(sig);
 	OQS_destroy();
 
